@@ -1,8 +1,9 @@
 from pathlib import Path
+from types import SimpleNamespace
 from typing import List, Tuple
 
 import astropy.units as u
-import matplotlib
+import matplotlib as mpl
 import matplotlib.lines as mlines
 import matplotlib.patches as patches
 import matplotlib.pyplot as plt
@@ -16,6 +17,7 @@ from . import io
 from .colors import get_colorlist
 from .utils import (
     _axplot,
+    get_labels,
     get_plot_layout,
     get_prefix,
     read_yaml_to_namespace,
@@ -24,7 +26,7 @@ from .utils import (
 
 quantity_support()
 plt.style.use(["science", "vibrant"])
-matplotlib.rcParams.update(
+mpl.rcParams.update(
     {
         "font.family": "serif",
         "image.resample": False,
@@ -41,25 +43,83 @@ COLUMNS = read_yaml_to_namespace(Path(__file__).parent / "config" / "columns.yam
 COLORS = plt.rcParams["axes.prop_cycle"].by_key()["color"]
 
 
-def get_label(column: str, unit: str) -> str:
-    """Sets the label."""
+# TODO: Switch this to an only column based approach
+def get_label(column: SimpleNamespace, unit: str) -> str:
+    """Gets an axis label from column and unit.
+
+    Parameters
+    ----------
+    column : str
+        The column of the extension.
+    unit : str
+        The unit of the column.
+
+    Returns
+    -------
+    ylabel : str
+        The y-axis label.
+    """
     # HACK: Very specific case, better solution possible?
-    unit = unit.replace(r"\mu m", r"\si{\micro\metre}")
+    unit = unit.replace(r"\mu m", r"\unit{\micro\metre}")
     return f"$ {column.label} $ ({unit})"
 
 
+# TODO: Makes this function irrelevant.
+def get_stations(
+    hdul: fits.HDUList, extension: str, version: int | None = None
+) -> List[str]:
+    """Gets the station names from the hdul for an extension.
+
+    Parameters
+    ----------
+    hdul : fits.HDUList
+    extension : str
+        The extension for which the stations are to be determined.
+    version : int, optional
+        The extension version ("EXTVER").
+
+    Returns
+    -------
+    stations : list of str
+    """
+    sta_index = io.get(hdul, f"{extension}:{version}.sta_index")
+    sta_index_to_name = dict(
+        zip(
+            io.get(hdul, "oi_array.sta_index").tolist(),
+            io.get(hdul, "oi_array.sta_name"),
+        )
+    )
+    return list(
+        map(lambda x: "-".join(x), np.vectorize(sta_index_to_name.get)(sta_index))
+    )
+
+
 def get_baseline_and_angle(
-    hdul, ext: str, index: int | None = None
+    hdul, extension: str, version: int | None = None
 ) -> Tuple[u.Quantity, u.Quantity]:
-    """Computes baseline and angle from (u, v)-coordinates of an extension."""
-    if "vis" in ext:
-        x = io._get_column(hdul, ext, "ucoord", index)
-        y = io._get_column(hdul, ext, "vcoord", index)
-    elif "t3" in ext:
+    """Computes baseline and angle from (u, v)-coordinates of an extension.
+
+    Parameters
+    ----------
+    hdul : fits.HDUList
+    extension : str
+        The extension for which the stations are to be determined.
+    version : int, optional
+        The extension version ("EXTVAR").
+
+    Returns
+    -------
+    baseline : astropy.units.Quantity
+    angle : astropy.units.Quantity
+    """
+    if "vis" in extension:
+        x = io.get(hdul, f"{extension}.ucoord", version)
+        y = io.get(hdul, f"{extension}.vcoord", version)
+    elif "t3" in extension:
         x, y = [], []
         for i in range(1, 3):
-            x.append(io._get_column(hdul, ext, f"u{i}coord", index))
-            y.append(io._get_column(hdul, ext, f"v{i}coord", index))
+            x.append(io.get(hdul, f"u{i}coord", version))
+            y.append(io.get(hdul, f"v{i}coord", version))
 
         x.append(x[0] + x[1])
         y.append(y[0] + y[1])
@@ -69,7 +129,7 @@ def get_baseline_and_angle(
         ...
 
     baseline, angle = np.hypot(x, y), np.arctan2(x, y)
-    if "t3" in ext:
+    if "t3" in extension:
         index_longest = np.argmax(baseline, axis=0)
         baseline = [b[i] for i, b in zip(index_longest, baseline.T)]
         angle = [a[i] for i, a in zip(index_longest, angle.T)]
@@ -78,10 +138,27 @@ def get_baseline_and_angle(
 
 
 def get_spatial_frequency(
-    wl: u.Quantity, baseline: u.Quantity, unit: str = "Mlam"
+    wavelength: u.Quantity, baseline: u.Quantity, unit: str = "Mlam"
 ) -> u.Quantity:
-    """Computes spatial frequency from baseline (considering unit)."""
-    sp_freq = baseline.reshape(-1, 1) / wl
+    """Gets the spatial frequency from the baseline and wavelength
+    in the passed unit.
+
+    Parameters
+    ----------
+    wavelenght : astropy.units.Quantity
+        The wavelength.
+    baseline : astropy.units.Quantity
+        The baseline.
+    unit : str, optional
+        The unit in which to return the spatial frequency.
+        Avaiable units are "cycles/mas", "cycles/arcsec", "Mlam" and "none".
+        Default is "Mlam" (mega lambda).
+
+    Returns
+    -------
+    spatial_frequency : astropy.units.Quantity
+    """
+    spatial_frequency = baseline.reshape(-1, 1) / wavelength
     match unit:
         case "cycles/mas":
             factor = u.mas.to(u.rad)
@@ -92,9 +169,10 @@ def get_spatial_frequency(
         case _:
             factor = 1
 
-    return factor * sp_freq
+    return factor * spatial_frequency
 
 
+# TODO: Replace all column queries here (with the better get -> Also for the units).
 class Oifits(Axes):
     """Class derived from matplotlib.axes.Axes that allows easy plotting of oifits data"""
 
@@ -114,6 +192,7 @@ class Oifits(Axes):
 
     # TODO: Implement the tracks again.
     # TODO: Implement here indicies like GRAVITY needs them
+    # TODO: Add the version here
     def uv(
         self,
         hduls: List[fits.HDUList],
@@ -122,6 +201,7 @@ class Oifits(Axes):
         overlay: str = "station",
         isolines: bool = True,
         tracks: bool = False,
+        version: int | None = None,
         **kwargs,
     ) -> Axes:
         """Plots the uv coverage.
@@ -133,50 +213,60 @@ class Oifits(Axes):
             A list of HDULists with model data to plot. The model hduls have to
             be in the same order as the hduls.
         color_by : str, optional
-            The coloring of the uv points. Can be "file"/"date", "instrument", or "station".
-            Default is "date" (i.e. the "file").
+            The coloring of the uv points. Can be "file"/"date", "instrument",
+            or "baseline"/"station". Default is "date" (i.e. the "file").
         overlay : str, optional
             If the uv points should be overlayed with something.
-            Possible options are "number" (for a file based numbering), "station" for the station names,
-            or "visamp", "vis2data" for plots of the visibilities or squared visibilities.
-            In case of "visamp" it will display the "visphi" on the mirrored side.
-            Default is "station".
+            Possible options are "number" (for a file based numbering), "station"
+            for the station names, or "visamp", "vis2data" for plots of the
+            visibilities or squared visibilities. In case of "visamp" it will display
+            the "visphi" on the mirrored side. Default is "station".
+        isolines : bool, optional
+        tracks : bool, optional
+        version : int, optional
+            The version of the extension ("EXTVAR").
+            Default is None (i.e. it proccurs the first extension).
         legend : bool, optional
             If the legend should be displayed.
 
+        Returns
+        -------
+        matplotlib.axes.Axes
         """
         column_x, column_y = COLUMNS.u, COLUMNS.v
         match color_by:
             case "instrument":
-                instruments = [
-                    io._get_header_entry(hdul, "instrument") for hdul in hduls
-                ]
+                instruments = [io.get(hdul, "instrumen") for hdul in hduls]
                 legend_labels = []
                 for entry in instruments:
                     if entry not in legend_labels:
                         legend_labels.append(entry)
             case "date" | "file" | _:
-                legend_labels = [io._get_header_entry(hdul, "date") for hdul in hduls]
+                legend_labels = [
+                    io.get(hdul, "date-obs").split("T")[0] for hdul in hduls
+                ]
 
         colors = dict(zip(legend_labels, COLORS))
         inset_amps, inset_phis = [], []
-        file_letters = io.get_labels(hduls)
+        file_letters = get_labels(hduls)
         for file_index, hdul in enumerate(hduls):
-            ucoord = io._get_column(hdul, "oi_vis2", "ucoord")
-            vcoord = io._get_column(hdul, "oi_vis2", "vcoord")
+            ucoord = io.get(hdul, "oi_vis2.ucoord")
+            vcoord = io.get(hdul, "oi_vis2.vcoord")
 
-            if color_by == "station":
+            if color_by in ["baseline", "station"]:
                 color = COLORS[: len(ucoord)] * 2
             else:
-                color = colors[io._get_header_entry(hdul, color_by)]
+                key = io.get(hdul, "date-obs" if "date" else "instrume")
+                color = colors[key.split("T")[0] if "T" in key else key]
 
-            line = self.scatter(
+            sc = self.scatter(
                 u.Quantity([ucoord, -ucoord]),
                 u.Quantity([vcoord, -vcoord]),
                 s=kwargs.get("markersize", 5),
                 marker="x",
                 color=color,
             )
+            plt.draw()
 
             labels = []
             if overlay in ["number", "station"]:
@@ -186,7 +276,7 @@ class Oifits(Axes):
                         for i in np.arange(ucoord.size)
                     ]
                 elif overlay == "station":
-                    labels = io._get_stations(hdul, "vis2")
+                    labels = get_stations(hdul, "vis2")
 
                 for label_index, label in enumerate(labels):
                     self.annotate(
@@ -198,97 +288,96 @@ class Oifits(Axes):
                         color="black",
                     )
             elif any(x in overlay for x in ["visamp", "vis2data"]):
-                # TODO: Rewrite this with better globals (e.g. that have "oi_vis" info)
-                x = io._get_column(hdul, "oi_wavelength", "eff_wave")
+                x = io.get(hdul, "eff_wave", unit=True)
                 xlims = (np.min(x), np.max(x))
                 yphi, yphierr, ymodelphi = None, None, None
                 if "visamp" in overlay:
-                    y = io._get_column(hdul, "oi_vis", "visamp", masked=True)
-                    yerr = io._get_column(hdul, "oi_vis", "visamperr", masked=True)
+                    y = io.get(hdul, "visamp")
+                    yerr = io.get(hdul, "visamperr")
                     if "only" not in overlay:
-                        yphi = io._get_column(hdul, "oi_vis", "visphi", masked=True)
-                        yphierr = io._get_column(
-                            hdul, "oi_vis", "visphierr", masked=True
-                        )
+                        yphi = io.get(hdul, "visphi")
+                        yphierr = io.get(hdul, "visphierr")
                     if model_hduls is not None:
-                        ymodel = io._get_column(
-                            model_hduls[file_index], "oi_vis", "visamp", masked=True
-                        )
-                        ymodelphi = io._get_column(
-                            model_hduls[file_index], "oi_vis", "visphi", masked=True
-                        )
+                        ymodel = io.get(model_hduls[file_index], "visamp")
+                        ymodelphi = io.get(model_hduls[file_index], "visphi")
                 else:
-                    # TODO: Rewrite this with better globals (e.g. that have "oi_vis" info)
-                    y = io._get_column(hdul, "oi_vis2", "vis2data", masked=True)
-                    yerr = io._get_column(hdul, "oi_vis2", "vis2err", masked=True)
+                    y = io.get(hdul, "vis2data")
+                    yerr = io.get(hdul, "vis2err")
                     if model_hduls is not None:
-                        ymodel = io._get_column(
-                            model_hduls[file_index], "oi_vis2", "vis2data", masked=True
-                        )
+                        ymodel = io.get(model_hduls[file_index], "vis2data")
+
+                kwargs_inset = {
+                    "inset_width": kwargs.pop("inset_width", 20),
+                    "inset_height": kwargs.pop("inset_height", 20),
+                    "show_axis": kwargs.pop("show_axis", False),
+                }
 
                 # TODO: At some point replace this with the `visualise` method
-                for amp_index, amp in enumerate(y):
-                    inset_kwargs = {
-                        "color": line[0].get_color(),
-                        "inset_width": kwargs.pop("inset_width", 20),
-                        "inset_height": kwargs.pop("inset_height", 20),
-                        "show_axis": kwargs.pop("show_axis", False),
-                        "linewidth": kwargs.pop("linewidth", 0.5),
+                for index_amp, amp in enumerate(y):
+                    kwargs_plot = {
+                        "color": sc.get_facecolor()[index_amp],
                         "errorbar": kwargs.pop("errorbar", True),
                     }
-                    inset_amp = _axplot(
+                    ax_amp = _axplot(
                         self,
                         x,
-                        amp,
-                        yerr[amp_index],
-                        inset_point=(ucoord[amp_index], vcoord[amp_index]),
-                        **inset_kwargs,
+                        amp.value,
+                        yerr[index_amp].value,
+                        inset_point=(ucoord[index_amp].value, vcoord[index_amp].value),
+                        kwargs_inset=kwargs_inset,
+                        **kwargs_plot,
                     )
-                    inset_amp.axhline(0, color="grey", linestyle="--", linewidth=0.35)
+                    ax_amp.axhline(0, color="grey", linestyle="--", linewidth=0.35)
                     if model_hduls is not None:
-                        inset_amp.plot(
+                        ax_amp.plot(
                             x,
-                            ymodel[amp_index],
+                            ymodel[index_amp].value,
                             color="black",
-                            alpha=0.8,
-                            linewidth=0.23,
+                            linewidth=0.30,
                             zorder=5,
                         )
 
-                    inset_amp.set_xlim(xlims)
-                    inset_amps.append(inset_amp)
+                    ax_amp.set_xlim(xlims)
+                    inset_amps.append(ax_amp)
                     if yphi is not None:
-                        inset_phi = _axplot(
+                        ax_phi = _axplot(
                             self,
                             x,
-                            yphi[amp_index],
-                            yphierr[amp_index],
-                            inset_point=(-ucoord[amp_index], -vcoord[amp_index]),
-                            **inset_kwargs,
+                            yphi[index_amp],
+                            yphierr[index_amp],
+                            inset_point=(
+                                -ucoord[index_amp].value,
+                                -vcoord[index_amp].value,
+                            ),
+                            kwargs_inset=kwargs_inset,
+                            **kwargs_plot,
                         )
                         if ymodelphi is not None:
-                            inset_phi.plot(
+                            ax_phi.plot(
                                 x,
-                                ymodelphi[amp_index],
+                                ymodelphi[index_amp].value,
                                 color="black",
-                                alpha=0.8,
-                                linewidth=0.23,
+                                linewidth=0.3,
                                 zorder=5,
                             )
-                        inset_phi.axhline(
-                            0, color="grey", linestyle="--", linewidth=0.35
+                        ax_phi.axhline(0, color="grey", linestyle="--", linewidth=0.35)
+                        _axplot(
+                            ax_phi,
+                            x,
+                            y[index_amp].value,
+                            yerr[index_amp].value,
+                            kwargs_inset=kwargs_inset,
+                            **kwargs_inset,
                         )
-                        self = _axplot(
-                            self, x, y[amp_index], yerr[amp_index], **inset_kwargs
-                        )
-                        inset_phi.set_xlim(xlims)
-                        inset_phis.append(inset_phi)
+                        ax_phi.set_xlim(xlims)
+                        inset_phis.append(ax_phi)
 
         if inset_amps:
             amp_bounds = np.array([amp_ax.get_ylim() for amp_ax in inset_amps])
             amp_lims = np.min(amp_bounds[:, 0]), np.max(amp_bounds[:, 1])
             [amp_ax.set_ylim(amp_lims) for amp_ax in inset_amps]
 
+        # TODO: Finihs this
         if inset_phis:
             phi_bounds = np.array([phi_ax.get_ylim() for phi_ax in inset_phis])
             # TODO: Make it so that the user can input their own lims
@@ -345,32 +434,37 @@ class Oifits(Axes):
         xname: str,
         yname: str,
         cname: str | None = None,
-        index: int | None = None,
+        version: int | None = None,
         **kwargs,
-    ):
+    ) -> None:
         """Plots all the specified observables in a collage.
 
         Parameters
         ----------
+        hduls : astropy.io.fits.HDUList or list of astropy.io.fits.HDUList
+        xname : str
+        yname : str
+        cname : str, optional
+        version : int, optional
         """
         self.xtype, self.ytype, self.cname = xname, yname, cname
         column_x, column_y = getattr(COLUMNS, xname), getattr(COLUMNS, yname)
         for hdul in hduls:
-            if io._get_hdu(hdul, ext := column_y.ext, index) is None:
+            if io.get(hdul, f"{(extension := column_y.ext)}:{version}") is None:
                 continue
 
-            wl = io._get_column(hdul, "oi_wavelength", "eff_wave", index, unit=True)
-            ydata = io._get_column(hdul, ext, yname, index, masked=True)
-            yerr = io._get_column(hdul, ext, column_y.err, index, masked=True)
+            wl = io.get(hdul, f"{version}.eff_wave")
+            ydata = io.get(hdul, f"{version}.{yname}")
+            yerr = io.get(hdul, f"{version}.{column_y.err}")
             match xname:
                 case "length":
-                    xdata, _ = get_baseline_and_angle(hdul, ext, index)
+                    xdata, _ = get_baseline_and_angle(hdul, f"{extension}:{version}")
                 case "spa_freq":
                     xdata = get_spatial_frequency(
-                        wl, get_baseline_and_angle(hdul, ext, index)[0]
+                        wl, get_baseline_and_angle(hdul, f"{extension}:{version}")[0]
                     )
                 case "pa":
-                    _, xdata = get_baseline_and_angle(hdul, ext, index)
+                    _, xdata = get_baseline_and_angle(hdul, extension, version)
                 case "eff_wave" | _:
                     xdata = wl.to(f"{get_prefix(wl.min())}{wl.decompose().unit}")
 
@@ -388,10 +482,10 @@ class Oifits(Axes):
         # TODO: Implement error if the types are different (corr vs. abs for multiple files)
         match yname:
             case "visamp":
-                amptype = io._get_header_entry(hduls[0], "amptyp", column_y.ext)
+                amptype = io.get(hduls[0], "amptyp")
                 column_y = column_y.corr if "corr" in amptype else column_y.abs
             case "visphi":
-                phitype = io._get_header_entry(hduls[0], "phityp", column_y.ext)
+                phitype = io.get(hduls[0], "phityp")
                 column_y = column_y.diff if "diff" in phitype else column_y.abs
         self.axis_presets(column_x, column_y, **kwargs)
         return self
@@ -408,6 +502,7 @@ def _vs_spf(
     number: bool = False,
     legend: bool = True,
     transparent: bool = False,
+    version: int | None = None,
     savefig: Path | None = None,
     **kwargs,
 ) -> None:
@@ -430,10 +525,13 @@ def _vs_spf(
         If the plots should be numbered.
     transparent : bool, optional
         If the plots should be transparent.
+    version : int, optional
+        The version of the extension ("EXTVAR") from which the key is to be gotten.
+        Default is None (i.e. it proccurs the first extension).
     savefig : Path, optional
         The save file for the plots.
     """
-    wls, stations, labels, file_letters = [], [], [], io.get_labels(hduls)
+    wavelengths, stations, labels, file_letters = [], [], [], get_labels(hduls)
     spfs, psis, vals, errs, model_vals = [], [], [], [], []
 
     extension, column, amptype = EXTENSIONS[observable], COLUMNS[observable], ""
@@ -445,34 +543,34 @@ def _vs_spf(
             else:
                 amptype = "abs"
 
-        val = io._get_column(hdul, extension, observable, masked=True)
-        err = io._get_column(hdul, extension, column["err"], masked=True)
+        val = io.get(hdul, observable)
+        err = io.get(hdul, column["err"])
         if model_hduls is not None:
-            model_vals.extend(io._get_column(model_hduls[index], extension, observable))
+            model_vals.extend(io.get(model_hduls[index], observable))
 
         label = [f"{file_letters[index]}.{i + 1}" for i in range(val.shape[0])]
-        sta_index = io._get_column(hdul, extension, "sta_index")
+        sta_index = io.get(hdul, f"{extension}.sta_index", version=version)
         sta_index_to_name = dict(
             zip(
-                io._get_column(hdul, "oi_array", "sta_index").tolist(),
-                io._get_column(hdul, "oi_array", "sta_name"),
+                io.get(hdul, "oi_array.sta_inde", version=version).tolist(),
+                io.get(hdul, "oi_array.sta_name", version=version),
             )
         )
         station = list(
             map(lambda x: "-".join(x), np.vectorize(sta_index_to_name.get)(sta_index))
         )
-        wls.extend(
-            [io._get_column(hdul, "oi_wavelength", "eff_wave") for _ in range(len(val))]
+        wavelengths.extend(
+            [io.get(hdul, "eff_wave", version=version) for _ in range(len(val))]
         )
         if observable in ["visamp", "visphi", "vis2data"]:
-            x = io._get_column(hdul, extension, "ucoord")
-            y = io._get_column(hdul, extension, "vcoord")
+            x = io.get(hdul, f"{extension}.ucoord")
+            y = io.get(hdul, f"{extension}.vcoord")
         else:
             x1, x2 = map(
-                lambda x: io._get_column(hdul, "oi_t3", x), ["u1coord", "u2coord"]
+                lambda x: io.get(hdul, x, version=version), ["u1coord", "u2coord"]
             )
             y1, y2 = map(
-                lambda x: io._get_column(hdul, "oi_t3", x), ["v1coord", "v2coord"]
+                lambda x: io.get(hdul, x, version=version), ["v1coord", "v2coord"]
             )
             x123, y123 = np.array([x1, x2, x1 + x2]), np.array([y1, y2, y1 + y2])
 
@@ -505,20 +603,20 @@ def _vs_spf(
 
     for index, (ax, b, psi) in enumerate(zip(axarr.flat, spfs, psis)):
         line = ax.plot(
-            wls[index],
+            wavelengths[index],
             vals[index],
             color=label_colors[labels[index].split(".")[0]],
             label=rf"{stations[index]}, B={b:.2f} m, $\psi$={psi:.2f}$^\circ$",
         )
         ax.fill_between(
-            wls[index],
+            wavelengths[index],
             vals[index] + errs[index],
             vals[index] - errs[index],
             color=line[0].get_color(),
             alpha=0.5,
         )
         if model_vals:
-            ax.plot(wls[index], model_vals[index], color="black", label="Model")
+            ax.plot(wavelengths[index], model_vals[index], color="black", label="Model")
 
         # TODO: Switch this here to annotate?
         if number:
@@ -546,7 +644,7 @@ def _vs_spf(
 
     [ax.remove() if not ax.has_data() else ax.set_ylim(ylims) for ax in axarr.flat]
 
-    fig.supxlabel(r"$ \lambda (\si{\metre}) $", fontsize=16)
+    fig.supxlabel(r"$\lambda (\unit{\metre})$", fontsize=16)
     fig.supylabel(
         column["label"] if not amptype else column["label"][amptype], fontsize=16
     )
