@@ -8,6 +8,7 @@ from oiplot import orbit
     "mean_anomaly, eccentricity, expected", [(5, 0.1, 5.554589), (2, 0.99, 32.361007)]
 )
 def test_solve_eccentric_anomaly(mean_anomaly, eccentricity, expected):
+    """Test the eccentricity solve."""
     E = np.rad2deg(
         orbit.solve_eccentric_anomaly(np.deg2rad(mean_anomaly), eccentricity)
     )
@@ -15,10 +16,12 @@ def test_solve_eccentric_anomaly(mean_anomaly, eccentricity, expected):
 
 
 def test_compute_true_anomaly(mean_anomaly, eccentricity, expected):
+    """Test computation of the true anomaly."""
     E = np.rad2deg(
         orbit.solve_eccentric_anomaly(np.deg2rad(mean_anomaly), eccentricity)
     )
     true_anomaly = orbit.compute_true_anomaly(E, eccentricity)
 
+
 # def test_get_epoch(, ):
 #     ...
